import styled from "styled-components";

export type TableColumn<T> = {
  label: string | JSX.Element;
  accessKey?: keyof T;
  renderCell?: ({ row, idx }: { row: T; idx: number }) => JSX.Element;
  renderValue?: ({ row, idx }: { row: T; idx: number }) => any;
  isVisible?: boolean;
  className?: string;
  width?: string;
};

export type TableProps<T> = {
  columns: TableColumn<T>[];
  tableData: T[];
  isLoading?: boolean;
};

export const Table = <T extends {}>({
  columns,
  tableData,
  isLoading = false,
}: TableProps<T>) => {
  const renderTbody = () => {
    return tableData.map((row, rowIndex) => (
      <tr key={rowIndex} className="common-table__tbody__tr">
        {columns.map((column, columnIdx) => {
          if (column.isVisible === false) {
            return undefined;
          }

          const { accessKey, className, renderValue, renderCell } = column;

          let value = undefined;

          if (renderCell) {
            value = renderCell({ row, idx: rowIndex });
          } else if (renderValue) {
            value = renderValue({ row, idx: rowIndex });
          } else if (accessKey) {
            value = row[accessKey];
          }

          return (
            <td
              key={`${rowIndex}-${String(accessKey || columnIdx)}`}
              className={className}
            >
              {value}
            </td>
          );
        })}
      </tr>
    ));
  };

  const renderSkeleton = () => {
    return Array.from({ length: 5 }, (_, rowIndex) => (
      <tr className="common-table__tbody__skeleton-container">
        {columns.map((column, columnIdx) => (
          <td key={`${String(column.accessKey || columnIdx)}_${rowIndex}`}>
            <div className="skeleton-item" />
          </td>
        ))}
      </tr>
    ));
  };

  return (
    <Container id="common-table">
      <thead className="common-table__thead">
        <tr className="common-table__thead__tr">
          {columns.map((column, columnIndex) => {
            if (column.isVisible === false) {
              return undefined;
            }

            return (
              <th
                key={`${String(
                  column.accessKey || columnIndex
                )}_${columnIndex}`}
                className={column.className}
                style={{ width: column.width }}
              >
                {column.label}
              </th>
            );
          })}
        </tr>
      </thead>
      <tbody className="common-table__tbody">
        {isLoading ? renderSkeleton() : renderTbody()}
      </tbody>
    </Container>
  );
};

const Container = styled.table`
  width: 100%;
  table-layout: fixed;
  border-collapse: collapse;

  thead tr th {
    text-align: left;
  }

  tbody tr td {
    height: 100%;
    vertical-align: top;
  }

  .common-table__tbody__skeleton-container {
    width: 100%;

    .skeleton-item {
      width: 100%;
      height: 16px;
      background-color: lightgray;
      border-radius: 40px;
    }
  }
`;
